from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Amplitude.AmplitudeOdb import AmplitudeOdb
from ..BeamSectionProfile.BeamSectionProfileOdb import BeamSectionProfileOdb
from ..Canvas.Displayable import Displayable
from ..Filter.FilterOdb import FilterOdb
from ..Material.MaterialOdb import MaterialOdb
from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from .OdbPart import OdbPart
from .OdbStep import OdbStep
from .SectionCategory import SectionCategory


@abaqus_class_doc
class Odb(AmplitudeOdb, FilterOdb, MaterialOdb, BeamSectionProfileOdb, Displayable):
    """The Odb object is the in-memory representation of an output database (ODB) file.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name]
    """

    @abaqus_method_doc
    def Part(
        self,
        name: str,
        embeddedSpace: Literal[C.THREE_D, C.TWO_D_PLANAR, C.AXISYMMETRIC],
        type: Literal[C.DEFORMABLE_BODY, C.ANALYTIC_RIGID_SURFACE],
    ) -> OdbPart:
        """This method creates an OdbPart object. Nodes and elements are added to this object at a later stage.

        .. note::
            This function can be accessed by::

                session.odbs[name].Part

        Parameters
        ----------
        name
            A String specifying the part name.
        embeddedSpace
            A SymbolicConstant specifying the dimensionality of the Part object. Possible values are
            THREE_D, TWO_D_PLANAR, and AXISYMMETRIC.
        type
            A SymbolicConstant specifying the type of the Part object. Possible values are
            DEFORMABLE_BODY and ANALYTIC_RIGID_SURFACE.

        Returns
        -------
        OdbPart
            An OdbPart object.
        """
        self.parts[name] = odbPart = OdbPart(name, embeddedSpace, type)
        return odbPart

    @abaqus_method_doc
    def Step(
        self,
        name: str,
        description: str,
        domain: Literal[C.TIME, C.FREQUENCY, C.ARC_LENGTH, C.MODAL],
        timePeriod: float = 0,
        previousStepName: str = "",
        procedure: str = "",
        totalTime: float = -1.0,
    ) -> OdbStep:
        """This method creates an OdbStep object.

        .. note::
            This function can be accessed by::

                session.odbs[name].Step

        Parameters
        ----------
        name
            A String specifying the repository key.
        description
            A String specifying the step description.
        domain
            A SymbolicConstant specifying the domain of the step. Possible values are TIME,
            FREQUENCY, ARC_LENGTH, and MODAL.The type of OdbFrame object that can be created for
            this step is based on the value of the **domain** argument.
        timePeriod
            A Float specifying the time period of the step. **timePeriod** is required if
            **domain** = TIME; otherwise, this argument is not applicable. The default value is 0.0.
        previousStepName
            A String specifying the preceding step. If **previousStepName** is the empty string, the
            last step in the repository is used. If **previousStepName** is not the last step, this
            will result in a change to the **previousStepName** member of the step that was in that
            position. A special value 'Initial' refers to the internal initial model step and may be
            used exclusively for inserting a new step at the first position before any other
            existing steps. The default value is an empty string.
        procedure
            A String specifying the step procedure. The default value is an empty string. The
            following is the list of valid procedures:

            - `*ANNEAL`
            - `*BUCKLE`
            - `*COMPLEX FREQUENCY`
            - `*COUPLED TEMPERATURE-DISPLACEMENT`
            - `*COUPLED TEMPERATURE-DISPLACEMENT, CETOL`
            - `*COUPLED TEMPERATURE-DISPLACEMENT, STEADY STATE`
            - `*COUPLED THERMAL-ELECTRICAL, STEADY STATE`
            - `*COUPLED THERMAL-ELECTRICAL`
            - `*COUPLED THERMAL-ELECTRICAL, DELTMX`
            - `*DYNAMIC`
            - `*DYNAMIC, DIRECT`
            - `*DYNAMIC, EXPLICIT`
            - `*DYNAMIC, SUBSPACE`
            - `*DYNAMIC TEMPERATURE-DISPLACEMENT, EXPLICT`
            - `*ELECTROMAGNETIC, HIGH FREQUENCY, TIME HARMONIC`
            - `*ELECTROMAGNETIC, LOW FREQUENCY, TIME DOMAIN`
            - `*ELECTROMAGNETIC, LOW FREQUENCY, TIME DOMAIN, DIRECT`
            - `*ELECTROMAGNETIC, LOW FREQUENCY, TIME HARMONIC`
            - `*FREQUENCY`
            - `*GEOSTATIC`
            - `*HEAT TRANSFER`
            - `*HEAT TRANSFER, DELTAMX=__`
            - `*HEAT TRANSFER, STEADY STATE`
            - `*MAGNETOSTATIC`
            - `*MAGNETOSTATIC, DIRECT`
            - `*MASS DIFFUSION`
            - `*MASS DIFFUSION, DCMAX=`
            - `*MASS DIFFUSION, STEADY STATE`
            - `*MODAL DYNAMIC`
            - `*RANDOM RESPONSE`
            - `*RESPONSE SPECTRUM`
            - `*SOILS`
            - `*SOILS, CETOL/UTOL`
            - `*SOILS, CONSOLIDATION`
            - `*SOILS, CONSOLIDATION, CETOL/UTOL`
            - `*STATIC`
            - `*STATIC, DIRECT`
            - `*STATIC, RIKS`
            - `*STEADY STATE DYNAMICS`
            - `*STEADY STATE TRANSPORT`
            - `*STEADY STATE TRANSPORT, DIRECT`
            - `*STEP PERTURBATION, *STATIC`
            - `*SUBSTRUCTURE GENERATE`
            - `*USA ADDDED MASS GENERATION`
            - `*VISCO`
        totalTime
            A Float specifying the analysis time spend in all the steps previous to this step. The
            default value is −1.0.

        Returns
        -------
        OdbStep
            An OdbStep object.

        Raises
        ------
        ValueError
            previousStepName is invalid, If **previousStepName** is invalid.
        """
        self.steps[name] = odbStep = OdbStep(
            name,
            description,
            domain,
            timePeriod,
            previousStepName,
            procedure,
            totalTime,
        )
        return odbStep

    @abaqus_method_doc
    def SectionCategory(self, name: str, description: str) -> SectionCategory:
        """This method creates a SectionCategory object.

        .. note::
            This function can be accessed by::

                session.odbs[name].SectionCategory

        Parameters
        ----------
        name
            A String specifying the name of the category.
        description
            A String specifying the description of the category.

        Returns
        -------
        SectionCategory
            A SectionCategory object.
        """
        self.sectionCategories[name] = sectionCategory = SectionCategory(name, description)
        return sectionCategory



from __future__ import annotations

from typing_extensions import Self

from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXDrawable import FXDrawable
from .FXObject import FXObject

#: Default layout mode.
LAYOUT_NORMAL: int = hash("LAYOUT_NORMAL")

#: Pack on top side (default).
LAYOUT_SIDE_TOP: int = hash("LAYOUT_SIDE_TOP")

#: Pack on bottom side.
LAYOUT_SIDE_BOTTOM: int = hash("LAYOUT_SIDE_BOTTOM")

#: Pack on left side.
LAYOUT_SIDE_LEFT: int = hash("LAYOUT_SIDE_LEFT")

#: Pack on right side.
LAYOUT_SIDE_RIGHT: int = hash("LAYOUT_SIDE_RIGHT")

#: Matrix column is stretchable.
LAYOUT_FILL_COLUMN: int = hash("LAYOUT_FILL_COLUMN")

#: Matrix row is stretchable.
LAYOUT_FILL_ROW: int = hash("LAYOUT_FILL_ROW")

#: Stick on left (default).
LAYOUT_LEFT: int = hash("LAYOUT_LEFT")

#: Stick on right.
LAYOUT_RIGHT: int = hash("LAYOUT_RIGHT")

#: Center horizontally.
LAYOUT_CENTER_X: int = hash("LAYOUT_CENTER_X")

#: X fixed.
LAYOUT_FIX_X: int = hash("LAYOUT_FIX_X")

#: Stick on top (default).
LAYOUT_TOP: int = hash("LAYOUT_TOP")

#: Stick on bottom.
LAYOUT_BOTTOM: int = hash("LAYOUT_BOTTOM")

#: Center vertically.
LAYOUT_CENTER_Y: int = hash("LAYOUT_CENTER_Y")

#: Y fixed CAE: Copied from FOX 1.4.34 to support dockable tool bars.
LAYOUT_FIX_Y: int = hash("LAYOUT_FIX_Y")

#: Dock on same galley if it fits.
LAYOUT_DOCK_SAME: int = hash("LAYOUT_DOCK_SAME")

#: Dock on next galley LAYOUT_RESERVED_1 = 0x00000040, LAYOUT_RESERVED_2 = 0x00000080,
LAYOUT_DOCK_NEXT: int = hash("LAYOUT_DOCK_NEXT")

#: CAE End.
LAYOUT_RESERVED_1: int = hash("LAYOUT_RESERVED_1")

#: Width fixed.
LAYOUT_FIX_WIDTH: int = hash("LAYOUT_FIX_WIDTH")

#: height fixed
LAYOUT_FIX_HEIGHT: int = hash("LAYOUT_FIX_HEIGHT")

#: Minimum width is the default.
LAYOUT_MIN_WIDTH: int = hash("LAYOUT_MIN_WIDTH")

#: Minimum height is the default.
LAYOUT_MIN_HEIGHT: int = hash("LAYOUT_MIN_HEIGHT")

#: Stretch or shrink horizontally.
LAYOUT_FILL_X: int = hash("LAYOUT_FILL_X")

#: Stretch or shrink vertically.
LAYOUT_FILL_Y: int = hash("LAYOUT_FILL_Y")

#: Explicit placement.
LAYOUT_EXPLICIT: int = hash("LAYOUT_EXPLICIT")

#: Default is no frame.
FRAME_NONE: int = hash("FRAME_NONE")

#: Sunken border.
FRAME_SUNKEN: int = hash("FRAME_SUNKEN")

#: Raised border.
FRAME_RAISED: int = hash("FRAME_RAISED")

#: Thick border.
FRAME_THICK: int = hash("FRAME_THICK")

#: A groove or etched-in border.
FRAME_GROOVE: int = hash("FRAME_GROOVE")

#: A ridge or embossed border.
FRAME_RIDGE: int = hash("FRAME_RIDGE")

#: Simple line border.
FRAME_LINE: int = hash("FRAME_LINE")

#: Regular raised/thick border.
FRAME_NORMAL: int = hash("FRAME_NORMAL")

#: Default is each its own size.
PACK_NORMAL: int = hash("PACK_NORMAL")

#: Uniform height.
PACK_UNIFORM_HEIGHT: int = hash("PACK_UNIFORM_HEIGHT")

#: Uniform width.
PACK_UNIFORM_WIDTH: int = hash("PACK_UNIFORM_WIDTH")

#: Default.
BACKGROUND_NORMAL: int = hash("BACKGROUND_NORMAL")

#: Horizontal gradient background.
BACKGROUND_H_GRADIENT: int = hash("BACKGROUND_H_GRADIENT")

#: Vertical gradient background.
BACKGROUND_V_GRADIENT: int = hash("BACKGROUND_V_GRADIENT")

#: plain background
BACKGROUND_PLAIN: int = hash("BACKGROUND_PLAIN")


class FXWindow(FXDrawable):
    """Base class for all windows."""

    def __init__(self, p: FXComposite, opts: int = 0, x: int = 0, y: int = 0, w: int = 0, h: int = 0):
        """Constructor.

        Parameters
        ----------
        p : FXComposite

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def canFocus(self):
        """Return True if this window is a control capable of receiving the focus.

        Reimplemented in FXArrowButton, FXButton, FXCanvas, FXCheckButton, FXColorWell, FXDockHandler,
        FXIconList, FXImageView, FXList, FXMDIChild, FXMenuButton, FXMenuCascade, FXMenuCommand,
        FXMenuTitle, FXOption, FXOptionMenu, FXRadioButton, FXSlider, FXTabItem, FXTable, FXText,
        FXTextField, FXToggleButton, FXToolbarTab, FXTreeList, AFXBaseTable, AFXFloatSpinner,
        AFXFlyoutButton, AFXFlyoutItem, and AFXSlider.
        """

    def childAtIndex(self, index: int):
        """Return the child window at specified index, or NULL if the index is negative or out of range
        Reimplemented in AFXOptionTreeItem.

        Parameters
        ----------
        index : int
        """

    def containsChild(self, child: Self):
        """Return True if specified window is a child of this window.

        Parameters
        ----------
        child : Self
        """

    def create(self):
        """Create all of the server-side resources for this window.

        Reimplemented from FXId. Reimplemented in FXColorBar, FXColorSelector, FXColorWell, FXColorWheel,
        FXComboBox, FXComposite, FXDirBox, FXDirList, FXDockTitle, FXDriveBox, FXFileList, FXFontSelector,
        FXGLCanvas, FXGLViewer, FXGroupBox, FXHeader, FXIconList, FXImageView, FXLabel, FXList, FXListBox,
        FXMDIChild, FXMenuButton, FXMenuCaption, FXMenuCascade, FXProgressBar, FXMenuTitle, FXOptionMenu,
        FXPrintDialog, FXRootWindow, FXScrollWindow, FXShell, FXSpinner, FXStatusline, FXTabBar, FXTable,
        FXText, FXTextField, FXToggleButton, FXToolbarShell, FXTooltip, FXTopWindow, FXTreeList,
        FXTreeListBox, AFXManagerMenuPane, AFXMainWindow, AFXPromptArea, AFXBaseTable, AFXColorButton,
        AFXColorFlyout, AFXComboBox, AFXDialog, AFXFloatSpinner, AFXFlyoutButton, AFXListBox, AFXNote,
        AFXOptionTreeItem, AFXPrimFloatSpinner, AFXProgressBar, AFXSpinner, AFXTable, AFXTextField, and
        AFXVerticalAligner.
        """

    def destroy(self):
        """Destroy the server-side resources for this window.

        Reimplemented from FXId. Reimplemented in FXComboBox, FXComposite, FXDirBox, FXDirList, FXDriveBox,
        FXFileList, FXGLCanvas, FXListBox, FXMenuCascade, FXOptionMenu, FXRootWindow, FXTreeList,
        FXTreeListBox, AFXManagerMenuCascade, AFXColorFlyout, and AFXTable.
        """

    def detach(self):
        """Detach the server-side resources for this window.

        Reimplemented from FXId. Reimplemented in FXColorBar, FXColorWell, FXColorWheel, FXComboBox,
        FXComposite, FXDirBox, FXDirList, FXDockTitle, FXDriveBox, FXFileList, FXGLCanvas, FXGLViewer,
        FXGroupBox, FXHeader, FXIconList, FXImageView, FXLabel, FXList, FXListBox, FXMDIChild, FXMenuButton,
        FXMenuCaption, FXMenuCascade, FXProgressBar, FXMenuTitle, FXOptionMenu, FXRootWindow, FXStatusline,
        FXTable, FXText, FXToggleButton, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, AFXBaseTable,
        AFXColorFlyout, AFXFlyoutButton, AFXNote, and AFXTable.
        """

    def disable(self):
        """Disable the window from receiving mouse and keyboard events.

        Reimplemented in FXArrowButton, FXComboBox, FXGroupBox, FXLabel, FXListBox, FXMenuCaption,
        FXScrollCorner, FXSlider, FXSpinner, FXText, FXTextField, FXToolbarTab, FXTreeListBox,
        AFXAutoComputeGroup, AFXManagerMenuDB, AFXColorButton, AFXColorFlyout, AFXComboBox, AFXFloatSpinner,
        AFXFlyoutButton, AFXList, AFXListBox, AFXNote, AFXOptionTreeItem, AFXPrimFloatSpinner, AFXSlider,
        AFXSpinner, AFXTable, and AFXTextField.
        """

    def enable(self):
        """Enable the window to receive mouse and keyboard events.

        Reimplemented in FXArrowButton, FXComboBox, FXGroupBox, FXLabel, FXListBox, FXMenuCaption,
        FXScrollCorner, FXSlider, FXSpinner, FXText, FXTextField, FXToolbarTab, FXTreeListBox,
        AFXAutoComputeGroup, AFXManagerMenuDB, AFXColorButton, AFXColorFlyout, AFXComboBox, AFXFloatSpinner,
        AFXFlyoutButton, AFXList, AFXListBox, AFXNote, AFXOptionTreeItem, AFXPrimFloatSpinner, AFXSlider,
        AFXSpinner, AFXTable, and AFXTextField.
        """

    def forceRefresh(self):
        """Force a GUI update of this window and its children."""

    def getBackColor(self):
        """Get background color.

        Reimplemented in FXComboBox, and FXListBox.
        """

    def getCursorPosition(self):
        """Returns a sequence of (status, x, y, mouseButtonState) representing the relative location of the
        cursor in the widget."""

    def getDefaultHeight(self):
        """Return the default height of this window.

        Reimplemented in FX4Splitter, FXArrowButton, FXCheckButton, FXColorBar, FXColorWell, FXColorWheel,
        FXComboBox, FXComposite, FXDial, FXDockSite, FXDockTitle, FXDragCorner, FXFrame, FXGroupBox,
        FXHeader, FXHorizontalFrame, FXLabel, FXList, FXListBox, FXMDIDeleteButton, FXMDIRestoreButton,
        FXMDIMaximizeButton, FXMDIMinimizeButton, FXMDIWindowButton, FXMDIChild, FXMatrix, FXMenuButton,
        FXMenuCaption, FXMenuCommand, FXProgressBar, FXMenuSeparator, FXMenuTitle, FXOption, FXOptionMenu,
        FXPacker, FXPopup, FXRadioButton, FXRootWindow, FXScrollArea, FXScrollbar, FXHorizontalSeparator,
        FXVerticalSeparator, FXSlider, FXSpinner, FXSplitter, FXStatusbar, FXStatusline, FXSwitcher,
        FXTabBar, FXTabBook, FXTable, FXText, FXTextField, FXToggleButton, FXToolbar, FXToolbarGrip,
        FXToolbarShell, FXToolbarTab, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, FXVerticalFrame,
        AFXMainWindow, AFXToolbarGroup, AFXBaseTable, AFXList, AFXOptionTreeList, AFXPrimFloatSpinner,
        AFXProgressBar, AFXSlider, AFXTable, AFXTreeTable, and AFXVerticalAligner.
        """

    def getDefaultWidth(self):
        """Return the default width of this window.

        Reimplemented in FX4Splitter, FXArrowButton, FXCheckButton, FXColorBar, FXColorWell, FXColorWheel,
        FXComboBox, FXComposite, FXDial, FXDockSite, FXDockTitle, FXDragCorner, FXFrame, FXGroupBox,
        FXHeader, FXHorizontalFrame, FXLabel, FXList, FXListBox, FXMDIDeleteButton, FXMDIRestoreButton,
        FXMDIMaximizeButton, FXMDIMinimizeButton, FXMDIWindowButton, FXMDIChild, FXMatrix, FXMenuButton,
        FXMenuCaption, FXMenuCommand, FXProgressBar, FXMenuSeparator, FXMenuTitle, FXOption, FXOptionMenu,
        FXPacker, FXPopup, FXRadioButton, FXRootWindow, FXScrollArea, FXScrollbar, FXHorizontalSeparator,
        FXVerticalSeparator, FXSlider, FXSpinner, FXSplitter, FXStatusbar, FXStatusline, FXSwitcher,
        FXTabBar, FXTabBook, FXTable, FXText, FXTextField, FXToggleButton, FXToolbar, FXToolbarGrip,
        FXToolbarShell, FXToolbarTab, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, FXVerticalFrame,
        AFXMainWindow, AFXToolbarGroup, AFXBaseTable, AFXOptionTreeItem, AFXOptionTreeList,
        AFXPrimFloatSpinner, AFXProgressBar, AFXSlider, AFXTable, AFXTextField, AFXTreeTable, and
        AFXVerticalAligner.
        """

    def getFirst(self):
        """Return a pointer to this window's first child window , if any.

        Reimplemented in AFXOptionTreeItem.
        """

    def getHeightForWidth(self, givenwidth: int):
        """Return height for given width. Reimplemented in FXDockSite.

        Parameters
        ----------
        givenwidth : int
        """

    def getKey(self):
        """Return window key."""

    def getLast(self):
        """Return a pointer to this window's last child window, if any.

        Reimplemented in AFXOptionTreeItem.
        """

    def getLayoutHints(self):
        """Get layout hints for this window."""

    def getNext(self):
        """Return a pointer to the next (sibling) window, if any.

        Reimplemented in AFXOptionTreeItem.
        """

    def getOwner(self):
        """Return a pointer to the owner window.

        Reimplemented in AFXMenuCascade, AFXMenuCommand, AFXMenuPane, AFXMenuTitle, AFXToolbarGroup, and
        AFXToolboxGroup.
        """

    def getParent(self):
        """Return a pointer to the parent window.

        Reimplemented in AFXOptionTreeItem.
        """

    def getPrev(self):
        """Return a pointer to the previous (sibling) window , if any.

        Reimplemented in AFXOptionTreeItem.
        """

    def getRoot(self):
        """Return a pointer to the root window."""

    def getSelector(self):
        """Get the message identifier for this window."""

    def getShell(self):
        """Return a pointer to the shell window."""

    def getTarget(self):
        """Get the message target object for this window, if any."""

    def getWidthForHeight(self, givenheight: int):
        """Return width for given height. Reimplemented in FXDockSite.

        Parameters
        ----------
        givenheight : int
        """

    def getX(self):
        """Get this window's x-coordinate, in the parent's coordinate system."""

    def getY(self):
        """Get this window's y-coordinate, in the parent's coordinate system."""

    def grab(self, confineTo: Self | None = None):
        """Grab the mouse to this window; future mouse events will be reported to this window even while the
        cursor goes outside of this window.

        Parameters
        ----------
        confineTo : Self | None
        """

    def grabbed(self):
        """Return True if the window has been grabbed."""

    def hasFocus(self):
        """Return True if this window has the focus."""

    def hide(self):
        """Hide this window.

        Reimplemented in FXTopWindow, AFXManagerMenuDB, AFXMenuTitle, AFXToolbarGroup,
        AFXToolbarGroupRender, AFXToolbarGroupVisibility, AFXDialog, AFXFlyoutItem, AFXMessageDialog,
        AFXOptionTreeItem, and AFXProgressBar.
        """

    def indexOfChild(self, window: Self):
        """Return the index (starting from zero) of the specified child window, or -1 if the window is not a
        child or NULL.

        Parameters
        ----------
        window : Self
        """

    def isActive(self):
        """Return True if the window is active.

        Reimplemented in AFXToolbarGroup.
        """

    def isChildOf(self, window: Self):
        """Return True if specified window is this window's parent.

        Parameters
        ----------
        window : Self
        """

    def isDefault(self):
        """Return True if this is the default window."""

    def isEnabled(self):
        """Return True if this window is able to receive mouse and keyboard events."""

    def isInitial(self):
        """Return True if this is the initial default window."""

    def linkAfter(self, sibling: Self):
        """Relink this window after sibling in the window list.

        Parameters
        ----------
        sibling : Self
        """

    def linkBefore(self, sibling: Self):
        """Relink this window before sibling in the window list.

        Parameters
        ----------
        sibling : Self
        """

    def move(self, x: int, y: int):
        """Move this window to the specified position in the parent's coordinates. Reimplemented in FXMDIChild,
        FXRootWindow, and FXTopWindow.

        Parameters
        ----------
        x : int

        y : int
        """

    def numChildren(self):
        """Return the number of child windows for this window.

        Reimplemented in AFXOptionTreeItem.
        """

    def position(self, x: int, y: int, w: int, h: int):
        """Move and resize this window in the parent's coordinates. Reimplemented in FXIconList, FXMDIChild,
        FXRootWindow, FXText, and FXTopWindow.

        Parameters
        ----------
        x : int

        y : int

        w : int

        h : int
        """

    def recalc(self):
        """Mark this window's layout as dirty.

        Reimplemented in FXIconList, FXList, FXMDIClient, FXRootWindow, FXShell, FXTable, FXText,
        FXTreeList, AFXBaseTable, AFXSlider, and AFXTable.
        """

    def repaint(self, x: int, y: int, w: int, h: int):
        """If marked but not yet painted, paint the given area.

        Parameters
        ----------
        x : int

        y : int

        w : int

        h : int
        """

    def resize(self, w: int, h: int):
        """Resize this window to the specified width and height. Reimplemented from FXDrawable. Reimplemented in
        FXIconList, FXMDIChild, FXRootWindow, FXText, and FXTopWindow.

        Parameters
        ----------
        w : int

        h : int
        """

    def setBackColor(self, clr: FXColor):
        """Set window background color. Reimplemented in FXComboBox, and FXListBox.

        Parameters
        ----------
        clr : FXColor
        """

    def setCursorPosition(self, x: int, y: int):
        """Warp the cursor to the new position.

        Parameters
        ----------
        x : int

        y : int
        """

    def setFocus(self):
        """Move the focus to this window.

        Reimplemented in FXButton, FXColorWell, FXIconList, FXList, FXMenuCascade, FXMenuCommand,
        FXMenuTitle, FXOption, FXPopup, FXRootWindow, FXShell, FXTable, FXText, FXTextField, FXTopWindow,
        FXTreeList, AFXBaseTable, AFXFlyoutItem, and AFXTextField.
        """

    def setHeight(self, h: int):
        """Set the window height.

        Parameters
        ----------
        h : int
        """

    def setInitial(self, enable: bool = True):
        """Make this window the initial default window.

        Parameters
        ----------
        enable : bool
        """

    def setKey(self, k: int):
        """Change window key.

        Parameters
        ----------
        k : int
        """

    def setLayoutHints(self, lout: int):
        """Set layout hints for this window.

        Parameters
        ----------
        lout : int
        """

    def setSelector(self, sel: int):
        """Set the message identifier for this window.

        Parameters
        ----------
        sel : int
        """

    def setTarget(self, t: FXObject):
        """Set the message target object for this window.

        Parameters
        ----------
        t : FXObject
        """

    def setWidth(self, w: int):
        """Set the window width.

        Parameters
        ----------
        w : int
        """

    def setX(self, x: int):
        """Set this window's x-coordinate, in the parent's coordinate system.

        Parameters
        ----------
        x : int
        """

    def setY(self, y: int):
        """Set this window's y-coordinate, in the parent's coordinate system.

        Parameters
        ----------
        y : int
        """

    def show(self):
        """Show this window.

        Reimplemented in FXTooltip, FXTopWindow, AFXMenuTitle, AFXToolbarGroup, AFXToolbarGroupRender,
        AFXToolbarGroupVisibility, AFXDialog, AFXFileDialog, AFXMessageDialog, AFXOptionTreeItem,
        AFXProgressBar, and AFXSlider.
        """

    def shown(self):
        """Return True if the window is shown."""

    def translateCoordinatesTo(self, tox: int, toy: int, towindow: Self, fromx: int, fromy: int):
        """Translate coordinates from this window's coordinate space to towindow's coordinate space.

        Parameters
        ----------
        tox : int

        toy : int

        towindow : Self

        fromx : int

        fromy : int
        """

    def ungrab(self):
        """Release the mouse grab."""

    def update(self, x: int, y: int, w: int, h: int):
        """Mark the specified rectangle dirty, i.e. to be repainted.

        Parameters
        ----------
        x : int

        y : int

        w : int

        h : int
        """



from __future__ import annotations

from abaqus.Connector.ConnectorDamage import ConnectorDamage
from abaqus.Connector.ConnectorDamping import ConnectorDamping
from abaqus.Connector.ConnectorElasticity import ConnectorElasticity
from abaqus.Connector.ConnectorFailure import ConnectorFailure
from abaqus.Connector.ConnectorFriction import ConnectorFriction
from abaqus.Connector.ConnectorLock import ConnectorLock
from abaqus.Connector.ConnectorPlasticity import ConnectorPlasticity
from abaqus.Connector.ConnectorPotential import ConnectorPotential
from abaqus.Connector.ConnectorStop import ConnectorStop

__all__ = [
    "ConnectorDamage",
    "ConnectorDamping",
    "ConnectorElasticity",
    "ConnectorFailure",
    "ConnectorFriction",
    "ConnectorLock",
    "ConnectorPlasticity",
    "ConnectorPotential",
    "ConnectorStop",
]



from __future__ import annotations

from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXObject import FXObject
from .FXWindow import FXWindow

#: Horizontally oriented.
SCROLLBAR_HORIZONTAL: int = hash("SCROLLBAR_HORIZONTAL")

#: Vertically oriented.
SCROLLBAR_VERTICAL: int = hash("SCROLLBAR_VERTICAL")


class FXScrollbar(FXWindow):
    """The scroll bar is used when a document has a larger content than may be made visible.

    The range is the total size of the document, the page is the part of the document which is visible. The
    size of the scrollbar thumb is adjusted to give feedback of the relative sizes of each. The scroll bar
    may be manipulated by the left mouse (normal scrolling), right mouse (vernier or fine-scrolling), or
    middle mouse (same as the left mouse only the scroll position can hop to the place where the click is
    made). Finally, if the mouse sports a wheel, the scroll bar can be manipulated by means of the mouse
    wheel as well. Holding down the Control-key during wheel motion will cause the scrolling to go faster
    than normal. While moving the scroll bar, a message of type SEL_CHANGED will be sent to the target, and
    the message data will reflect the current position of type FXint. At the end of the interaction, the
    scroll bar will send a message of type SEL_COMMAND to notify the target of the final position.
    """

    def __init__(
        self,
        p: FXComposite,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = SCROLLBAR_VERTICAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Construct scroll bar.

        Parameters
        ----------
        p : FXComposite

        tgt : FXObject | None

        sel : int

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def getBorderColor(self):
        """Change border color."""

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXWindow.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXWindow.
        """

    def getHiliteColor(self):
        """Return highlight color."""

    def getLine(self):
        """Return line increment."""

    def getPage(self):
        """Return page size."""

    def getPosition(self):
        """Return scroll position."""

    def getRange(self):
        """Return content size range."""

    def getScrollbarStyle(self):
        """Change the scrollbar style."""

    def getShadowColor(self):
        """Return shadow color."""

    def setBorderColor(self, clr: FXColor):
        """Return border color.

        Parameters
        ----------
        clr : FXColor
        """

    def setHiliteColor(self, clr: FXColor):
        """Change highlight color.

        Parameters
        ----------
        clr : FXColor
        """

    def setLine(self, l: int):
        """Set scoll increment for line.

        Parameters
        ----------
        l : int
        """

    def setPage(self, p: int):
        """Set viewport page size.

        Parameters
        ----------
        p : int
        """

    def setPosition(self, p: int, notifyTgt: bool = False):
        """Change current scroll position.

        Parameters
        ----------
        p : int

        notifyTgt : bool
        """

    def setRange(self, r: int):
        """Set content size range.

        Parameters
        ----------
        r : int
        """

    def setScrollbarStyle(self, style: int):
        """Get the current scrollbar style.

        Parameters
        ----------
        style : int
        """

    def setShadowColor(self, clr: FXColor):
        """Change shadow color.

        Parameters
        ----------
        clr : FXColor
        """



from __future__ import annotations

from .FXComposite import FXComposite
from .FXIcon import FXIcon
from .FXMenuCaption import FXMenuCaption
from .FXObject import FXObject

#: Normal, unchecked state.
MENUSTATE_NORMAL: int = hash("MENUSTATE_NORMAL")

#: Checked with a checkmark.
MENUSTATE_CHECKED: int = hash("MENUSTATE_CHECKED")

#: Checked with a bullet.
MENUSTATE_RCHECKED: int = hash("MENUSTATE_RCHECKED")


class FXMenuCommand(FXMenuCaption):
    """The menu command widget is used to invoke a command in the application from a menu.

    Menu commands may reflect the state of the application by graying out, becoming hidden, or by a check
    mark or bullit.
    """

    def __init__(
        self,
        p: FXComposite,
        text: str,
        ic: FXIcon | None = None,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = 0,
    ):
        """Construct a menu command.

        Parameters
        ----------
        p : FXComposite

        text : str

        ic : FXIcon | None

        tgt : FXObject | None

        sel : int

        opts : int
        """

    def canFocus(self):
        """Yes it can receive the focus.

        Reimplemented from FXWindow.
        """

    def check(self):
        """Place checkmark next to text."""

    def checkRadio(self):
        """Place radio bullit next to text."""

    def getAccelText(self):
        """Return accelarator text."""

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXMenuCaption.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXMenuCaption.
        """

    def isChecked(self):
        """Return True if checked."""

    def isRadioChecked(self):
        """Return True if radio-checked."""

    def setAccelText(self, text: str):
        """Set accelerator text.

        Parameters
        ----------
        text : str
        """

    def uncheck(self):
        """Uncheck the item."""

    def uncheckRadio(self):
        """Uncheck radio bullit."""



"""
Print Contour Plots
-------------------

`viewerPrintContours.py` from `Printing a contour plot at the end of each step <https://help.3ds.com/2021/English/DSSIMULIA_Established/SIMACAECMDRefMap/simacmd-c-intexaprintcontour.htm?contextscope=all&id=01196185741b4c25ae212a9cc00dc138>`_.

Print a set of contour plots to .png files.

Run the following command before running this script:

.. code-block:: sh

    abaqus fetch job=viewer_tutorial

The following example script demonstrates how to produce and print a contour plot at the last frame of every step within an output database file. The example sets the appropriate contour limits so that all plots can be viewed within a fixed range.

Use the following command to retrieve the example script:

.. code-block:: sh

    abaqus fetch job=viewerPrintContours

The script does the following:

- Defines the contour limits function.
- Determines the final frame of every step within an output database file.
- Produces a contour plot at the final frame of every step.
- Prints the contour plot to a file.
"""

import visualization
from abaqus import *
from abaqusConstants import *

# Create a viewport for this example.

myViewport = session.Viewport(name="Print contour plot after each step", origin=(10, 10), width=150, height=100)

# Open the output database and associate it with the viewport.
# Then set the plot state to CONTOURS_ON_DEF

try:
    myOdb = visualization.openOdb(path="viewer_tutorial.odb")

except Exception as e:
    print("Error:", e)

myViewport.setValues(displayedObject=myOdb)


myViewport.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF,))

# Determine the number of steps in the output database.

mySteps = myOdb.steps
numSteps = len(mySteps)

# Set the maximum and minimum limits of the contour legend.

myViewport.odbDisplay.contourOptions.setValues(
    numIntervals=10, maxAutoCompute=OFF, maxValue=0.1, minAutoCompute=OFF, minValue=0.0
)

# Establish print preferences.

session.printOptions.setValues(vpBackground=OFF)
session.psOptions.setValues(orientation=LANDSCAPE)
myViewport.viewportAnnotationOptions.setValues(triad=OFF, title=OFF, state=OFF)
myViewport.odbDisplay.basicOptions.setValues(
    coordSystemDisplay=OFF,
)

# For each step, obtain the following:
#     1) The step key.
#     2) The number of frames in the step.
#     3) The increment number of the last frame in the step.
#

for i in range(numSteps):
    stepKey = mySteps.keys()[i]
    step = mySteps[stepKey]
    numFrames = len(step.frames)

    #   Go to the last frame.
    #   Display a contour plot.
    #   Display the step description and the increment number.

    myViewport.odbDisplay.setFrame(step=i, frame=numFrames - 1)
    myViewport.odbDisplay.display.setValues(plotState=(CONTOURS_ON_DEF,))

    # Remove white space from the step key and use the result
    # to name the file.

    fileName = stepKey.replace(" ", "")

    # Print the viewport to a file.

    session.printToFile(fileName, PNG, (myViewport,))



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

import inspect
import json
import os
import re
import sys
import typing

from packaging.version import Version
from sphinx.builders.html import StandaloneHTMLBuilder

# Environment variables
os.environ["ABQPY_MAKE_DOCS"] = "true"

import abqpy  # noqa

project = "abqpy"
copyright = "2022, WANG Hailin"
author = "WANG Hailin"

release = abqpy.__version__
rel = Version(release)
version, major = rel.base_version, rel.major
branch = major if not rel.is_prerelease else "latest"

sys.path.insert(0, os.path.abspath("../../src"))
sys.path.insert(0, os.path.abspath("./_ext"))

# For multiple languages
language = os.environ.get("LANGUAGE", os.environ.get("READTHEDOCS_LANGUAGE", "en"))
locale_dirs = [f"locale/{major}"]  # path is example but recommended.
gettext_compact = False  # optional.

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "autoclasstoc",
    "automembers",
    "changes",
    "classdocumenter",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.linkcode",
    "sphinx.ext.githubpages",
    "sphinx.ext.intersphinx",
    "sphinx.ext.mathjax",
    "sphinx.ext.napoleon",
    "sphinx_autodoc_typehints",
    "sphinx_copybutton",
    "sphinx_codeautolink",
    "sphinx_comments",
    "sphinx_design",
    "sphinx_gallery.gen_gallery",
    "sphinx_inline_tabs",
    "sphinx_togglebutton",
    "sphinx_toolbox.confval",
    "sphinx_toolbox.more_autodoc.overloads",
    "sphinxcontrib.programoutput",
    "hoverxref.extension",
    "myst_parser",
    "version",
    "autoapi.extension",
    "sphinx_immaterial",
]

# changes configuration
changes_write_to = "CHANGES.md"
changes_versions = [str(v) for v in range(int(major), 2015, -1)]

# sphinx-comments
comments_config = {
    "utterances": {
        "repo": "haiiliin/abqpy",
    }
}

# automembers configuration
automembers_autodoc_options = [
    ":members:",
    ":show-inheritance:",
]

# MyST configuration
myst_enable_extensions = [
    "colon_fence",
]

# AutoAPI configuration
autoapi_dirs = ["../../src/abqpy"]
autoapi_ignore = ["*_version.py"]
autoapi_options = [
    "members",
    "undoc-members",
    "private-members",
    "show-inheritance",
    "show-module-summary",
    "special-members",
    # 'imported-members',
]
autoapi_template_dir = "_autoapi_templates"

# Default behavior for code block concatenation for sphinx_codeautolink
codeautolink_concat_default = False

# Suppress warnings
suppress_warnings = [
    "app.add_directive",
]

intersphinx_mapping = {
    "jinjia2": ("https://jinja.palletsprojects.com/en/3.0.x/", None),
    "matplotlib": ("https://matplotlib.org/stable/", None),
    "numpy": ("https://numpy.org/doc/stable/", None),
    "pandas": ("https://pandas.pydata.org/pandas-docs/stable/", None),
    "pytest": ("https://pytest.org/en/stable/", None),
    "python": ("https://docs.python.org/3/", None),
    "readthedocs": ("https://docs.readthedocs.io/en/stable/", None),
    "scipy": ("https://docs.scipy.org/doc/scipy/", None),
}

# Hoverxref configuration
hoverxref_auto_ref = True
hoverxref_domains = ["py"]
hoverxref_roles = [
    "numref",
    "confval",
    "setting",
    "option",
    "doc",  # Documentation pages
    "term",  # Glossary terms
]
hoverxref_role_types = {
    "doc": "modal",  # for whole docs
    "mod": "modal",  # for Python Sphinx Domain
    "class": "tooltip",  # for Python Sphinx Domain
    "func": "tooltip",  # for Python Sphinx Domain
    "meth": "tooltip",  # for Python Sphinx Domain
    "attr": "tooltip",  # for Python Sphinx Domain
    "exc": "tooltip",  # for Python Sphinx Domain
    "obj": "tooltip",  # for Python Sphinx Domain
    "ref": "tooltip",  # for hoverxref_auto_ref config
    "confval": "tooltip",  # for custom object
    "term": "tooltip",  # for glossaries
    "numref": "tooltip",
    "param": "tooltip",
}
hoverxref_intersphinx = [
    "numpy",
    "pytest",
    "python",
    "readthedocs",
]

# Sphinx gallery configuration
sphinx_gallery_conf = {
    "examples_dirs": "../../examples",
    "gallery_dirs": "examples",
    "filename_pattern": "/.+\.py",
    "plot_gallery": False,
    "nested_sections": False,
}

# Show short type hints for user-defined classes and defaults for parameters
python_use_unqualified_type_names = True
autodoc_typehints_format = "short"
typehints_defaults = "comma"
typehints_document_rtype = False
autodoc_default_options = {
    "undoc-members": False,
}
maximum_signature_line_length = 90
add_module_names = False
autoclass_content = "class"

# Figure numbering
numfig = True

# True to convert the type definitions in the docstrings as references. Defaults to False.
napoleon_preprocess_types = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["locale/README.md", "_autoapi_templates"]

# preferred image types for HTML output
StandaloneHTMLBuilder.supported_image_types = ["image/svg+xml", "image/gif", "image/png", "image/jpeg"]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "sphinx_immaterial"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
READTHEDOCS = "READTHEDOCS" in os.environ
versions = ["latest"] + [str(v) for v in range(2024, 2015, -1)]
with open(os.path.join(os.path.dirname(__file__), "locale", "edit-urls.json")) as f:
    edit_urls = json.load(f)
html_theme_options = {
    "icon": {
        "repo": "fontawesome/brands/github",
        "edit": "material/file-edit-outline",
    },
    "site_url": "https://abqpy.readthedocs.io/" if READTHEDOCS else "https://haiiliin.github.io/abqpy/",
    "repo_url": "https://github.com/haiiliin/abqpy/",
    "repo_name": "abqpy",
    "edit_uri": f"edit/{major}/docs/source",
    "edit_url_pages": edit_urls[f"{major}"].get(language, {}),
    "globaltoc_collapse": True,
    "features": [
        "navigation.expand",
        "navigation.tabs",
        # "toc.integrate",
        "navigation.sections",
        # "navigation.instant",
        # "header.autohide",
        "navigation.top",
        # "navigation.tracking",
        # "search.highlight",
        "search.share",
        "toc.follow",
        "toc.sticky",
        "content.tabs.link",
        "announce.dismiss",
    ],
    "palette": [
        {
            "media": "(prefers-color-scheme: light)",
            "scheme": "default",
            "toggle": {
                "icon": "material/brightness-7",
                "name": "Switch to dark mode",
            },
        },
        {
            "media": "(prefers-color-scheme: dark)",
            "scheme": "slate",
            "toggle": {
                "icon": "material/brightness-4",
                "name": "Switch to light mode",
            },
        },
    ],
    # BEGIN: version_dropdown
    "version_dropdown": True,
    "version_info": [
        {
            "version": f"/{language}/{ver}" if READTHEDOCS else f"/abqpy/{language}/{ver}",
            "title": ver,
            "aliases": [],
        }
        for ver in versions
    ],
    # END: version_dropdown
    "languages": [
        {"name": alias, "link": f"/{lang}/{branch}" if READTHEDOCS else f"/abqpy/{lang}/{branch}", "lang": lang}
        for lang, alias in zip(("en", "zh_CN"), ("English", "简体中文"))
    ],
    "toc_title_is_page_title": True,
    # BEGIN: social icons
    "social": [
        {
            "icon": "fontawesome/brands/github",
            "link": "https://github.com/haiiliin/abqpy",
            "name": "Source on github.com",
        },
        {
            "icon": "fontawesome/brands/python",
            "link": "https://pypi.org/project/abqpy",
        },
    ],
    # END: social icons
}

# Logo
# html_logo = "_static/3ds-dark.svg"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# -- Options for LaTeX output -------------------------------------------------

# If true, add page references after internal references. This is very useful
# for printed copies of the manual. Default is False.
latex_show_pagerefs = True

# Control whether to display URL addresses. This is very useful for printed
# copies of the manual. The setting can have the following values:
# 'no' - do not display URLs (default)
# 'footnote' - display URLs in footnotes
# 'inline' - display URLs inline in parentheses
latex_show_urls = "footnote"

# If given, this must be the name of an image file (relative to the
# configuration directory) that is the logo of the docs. It is placed at the
# top of the title page. Default: None.
latex_logo = None

# The “theme” that the LaTeX output should use. It is a collection of settings
# for LaTeX output (ex. document class, top level sectioning unit and so on).
# As a built-in LaTeX themes, manual and howto are bundled.
# manual
# A LaTeX theme for writing a manual. It imports the report document class
# (Japanese documents use jsbook).
# howto
# A LaTeX theme for writing an article. It imports the article document class
# (Japanese documents use jreport rather). latex_appendices is available only for this theme.
# It defaults to 'manual'.
latex_theme = "manual"

latex_toplevel_sectioning = "part"
latex_engine = "xelatex"
latex_use_xindy = False
latex_elements = {
    "preamble": "\\usepackage[UTF8]{ctex}\n\\setcounter{tocdepth}{3}\n\\setcounter{secnumdepth}{5}",
    "printindex": "\\def\\twocolumn[#1]{#1}\\printindex",
}


# linkcode source
def linkcode_resolve(domain: str, info: dict[str, typing.Union[str, list[str]]]):
    """Resolve linkcode source

    Parameters
    ----------
    domain : str
        specifies the language domain the object is in
    info : dict[str, str | list[str]]
        a dictionary with the following keys guaranteed to be present (dependent on the domain)

        - py: module (name of the module), fullname (name of the object)
        - c: names (list of names for the object)
        - cpp: names (list of names for the object)
        - javascript: object (name of the object), fullname (name of the item)

    Returns
    -------
    source url of the object
    """
    if domain != "py":
        return None

    modname = info["module"]
    fullname = info["fullname"]

    filename = modname.replace(".", "/")
    baseurl = f"https://github.com/haiiliin/abqpy/blob/{major}/src/{filename}.py"

    submod = sys.modules.get(modname)
    if submod is None:
        return baseurl

    obj = submod
    for part in fullname.split("."):
        try:
            obj = getattr(obj, part)
        except Exception:
            return baseurl
    try:
        source, lineno = inspect.getsourcelines(obj)
    except TypeError:
        # Find source line for an attribute, the obj is None
        attr = fullname.split(".")[-1]
        obj = submod
        for part in fullname.split(".")[:-1]:
            try:
                obj = getattr(obj, part)
            except Exception:
                return baseurl
        source, lineno = inspect.getsourcelines(obj)
        attr_sources: list[str] = re.findall(rf"\n(    {attr}: [\w\W]+?)\n\n", "\n".join(source))
        if len(attr_sources) > 0:
            attr_source = attr_sources[0].splitlines()

            def find_line_number(string: str, text: list[str]):
                for line_number, line in enumerate(text):
                    if string in line:
                        return line_number

            index = find_line_number(attr_source[0], source)
            for row in range(index - 1, -1, -1):
                if source[row].startswith("    #: "):
                    attr_source.insert(0, source[row])
                else:
                    break
            lineno += find_line_number(attr_source[0], source)
            source = attr_source
    except Exception:
        return baseurl

    return baseurl + f"#L{lineno}-L{lineno + len(source) - 1}"



from __future__ import annotations


class AFXSequenceString:
    """This class supports parsing and modification of strings containing sequences of elements separated with
    some separator character."""

    def __init__(self):
        """Undefined copy constructor (this class has no copy semantics)."""

    def forceNumElements(self, num: int, fill: str):
        """Forces the content string to contain a tuple with the given number of elements.

        Parameters
        ----------
        num : int
            New number of elements.
        fill : str
            String to insert in empty spaces.
        """

    def getContentString(self):
        """Returns a string containing values of the sequence elements.

        Reimplemented in AFX2DArrayConstString.
        """

    def getElementSeparator(self):
        """Returns the element separator character."""

    def getLength(self, index: int):
        """Returns the length in characters of a sequence element.

        Parameters
        ----------
        index : int
            Element index.
        """

    def getNumElements(self):
        """Returns the number of elements in this sequence."""

    def getPosition(self, index: int):
        """Returns the position in the content string of the beginning character of the sequence element.

        Parameters
        ----------
        index : int
            Element index.
        """

    def getValue(self, index: int):
        """Returns the value of a sequence element.

        Parameters
        ----------
        index : int
            Element index.
        """

    def insert(self, index: int, numElements: int, val: str):
        """Inserts many copies of an element.

        Parameters
        ----------
        index : int
            Element index at which inserting begins.
        numElements : int
            Number of elements to insert.
        val : str
            Value for the new elements.
        """

    def isValidSequence(self):
        """Returns True if this object contains a valid sequence."""

    def remove(self, index: int, numElements: int):
        """Removes elements starting at the given index.

        Parameters
        ----------
        index : int
            Element index at which removal begins.
        numElements : int
            Number of elements to remove.
        """

    def setContentString(self, seqstr: str):
        """Resets all values for the sequence elements.

        Parameters
        ----------
        seqstr : str
            Sequence string with new values.
        """

    def setElementSeparator(self, sep: str):
        """Sets the element separator character.

        Parameters
        ----------
        sep : str
            Separator character.
        """

    def setLength(self, index: int, length: int):
        """Sets the length of the sequence element.

        Parameters
        ----------
        index : int
            Element index.
        length : int
            New length (in characters).
        """

    def setPosition(self, index: int, position: int):
        """Sets the position of the sequence element.

        Parameters
        ----------
        index : int
            Element index.
        position : int
            New position within the string.
        """

    def setValue(self, index: int, value: str, replaceAll: bool = False):
        """Sets the value of a sequence element.

        Parameters
        ----------
        index : int
            Element index.
        value : str
            New value.
        replaceAll : bool
            If False (default), leading and trailing spaces will be preserved, otherwise, all space between separators will be replaced with the new value.
        """

    def trimWhiteSpace(self, index: int):
        """Adjusts the position and length of the element to trim leading and trailing white spaces.

        Parameters
        ----------
        index : int
            Element index.
        """



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ....Material.Ratios import Ratios


@abaqus_class_doc
class MoistureSwelling:
    r"""The MoistureSwelling object defines moisture-driven swelling.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].moistureSwelling
            import odbMaterial
            session.odbs[name].materials[name].moistureSwelling

        The table data for this object are:

        - Volumetric moisture swelling strain, :math:`\varepsilon^{m s}`.
        - Saturation, :math:`s`. This value must lie in the range :math:`0.0 \leq s \leq 1.0`.

        The corresponding analysis keywords are:

        - MOISTURE SWELLING
    """

    #: A Ratios object.
    ratios: Ratios = Ratios(((),))

    @abaqus_method_doc
    def __init__(self, table: tuple):
        """This method creates a MoistureSwelling object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].MoistureSwelling
                session.odbs[name].materials[name].MoistureSwelling

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.

        Returns
        -------
        MoistureSwelling
            A MoistureSwelling object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the MoistureSwelling object."""
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .ContactProperty import ContactProperty


@abaqus_class_doc
class ActuatorSensorProp(ContactProperty):
    """The ActuatorSensorProp object is an interaction property that defines the properties referred to by an
    ActuatorSensor object. The ActuatorSensorProp object is derived from the InteractionProperty object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]

        The corresponding analysis keywords are:

        - UEL PROPERTY
    """

    #: A String specifying the interaction property repository key.
    name: str

    #: A sequence of Floats specifying the PROPS array used by user subroutine UEL. The default
    #: value is an empty sequence.
    realProperties: tuple = ()

    #: A sequence of Ints specifying the JPROPS array used by user subroutine UEL. The default
    #: value is an empty sequence.
    integerProperties: tuple = ()

    @abaqus_method_doc
    def __init__(self, name: str, realProperties: tuple = (), integerProperties: tuple = ()):
        """This method creates an ActuatorSensorProp object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ActuatorSensorProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        realProperties
            A sequence of Floats specifying the PROPS array used by user subroutine UEL. The default
            value is an empty sequence.
        integerProperties
            A sequence of Ints specifying the JPROPS array used by user subroutine UEL. The default
            value is an empty sequence.

        Returns
        -------
        ActuatorSensorProp
            An ActuatorSensorProp object.
        """
        super().__init__(name)

    @abaqus_method_doc
    def setValues(self, realProperties: tuple = (), integerProperties: tuple = ()):
        """This method modifies the ActuatorSensorProp object.

        Parameters
        ----------
        realProperties
            A sequence of Floats specifying the PROPS array used by user subroutine UEL. The default
            value is an empty sequence.
        integerProperties
            A sequence of Ints specifying the JPROPS array used by user subroutine UEL. The default
            value is an empty sequence.
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Adaptivity.AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..Adaptivity.AdaptiveMeshDomain import AdaptiveMeshDomain
from ..BoundaryCondition.BoundaryConditionState import BoundaryConditionState
from ..Load.LoadCase import LoadCase
from ..Load.LoadState import LoadState
from ..PredefinedField.PredefinedFieldState import PredefinedFieldState
from ..StepMiscellaneous.Control import Control
from ..StepMiscellaneous.SolverControl import SolverControl
from ..StepOutput.DiagnosticPrint import DiagnosticPrint
from ..StepOutput.FieldOutputRequestState import FieldOutputRequestState
from ..StepOutput.HistoryOutputRequestState import HistoryOutputRequestState
from ..StepOutput.Monitor import Monitor
from ..StepOutput.Restart import Restart
from ..UtilityAndView.abaqusConstants import (
    AC_ON,
    ALL,
    DEFAULT,
    DISPLACEMENT,
    OFF,
    ON,
    SOLVER_DEFAULT,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .AnalysisStep import AnalysisStep


@abaqus_class_doc
class FrequencyStep(AnalysisStep):
    """The FrequencyStep object is used to perform eigenvalue extraction to calculate the natural frequencies
    and corresponding mode shapes of a system. The FrequencyStep object is derived from the AnalysisStep object.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]

        The corresponding analysis keywords are:

        - FREQUENCY
        - STEP
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A SymbolicConstant specifying the eigensolver. Possible values are LANCZOS, SUBSPACE,
    #: and AMS.The following optional arguments are ignored unless **eigensolver** = LANCZOS:
    #: **blockSize**, **maxBlocks**, **normalization**, **propertyEvaluationFrequency**.The following
    #: optional arguments are ignored unless **eigensolver** = LANCZOS or AMS: **minEigen**,
    #: **maxEigen**, and **acousticCoupling**.The following optional arguments are ignored unless
    #: **eigensolver** = AMS: **projectDamping**, **acousticRangeFactor**,
    #: **substructureCutoffMultiplier**, **firstCutoffMultiplier**, **secondCutoffMultiplier**,
    #: **residualModeRegion**, **regionalModeDof**, and **limitSavedEigenvectorRegion**.
    eigensolver: SymbolicConstant

    #: The SymbolicConstant ALL or an Int specifying the number of eigenvalues to be calculated
    #: or ALL. The default value is ALL.
    numEigen: SymbolicConstant = ALL

    #: A Float specifying the shift point in cycles per time. The default value is 0.0.
    shift: float = 0

    #: None or a Float specifying the minimum frequency of interest in cycles per time. The
    #: default value is None.
    minEigen: float | None = None

    #: None or a Float specifying the maximum frequency of interest in cycles per time. The
    #: default value is None.
    maxEigen: float | None = None

    #: None or an Int specifying the number of vectors used in the iteration. The default is
    #: the minimum of (2*n*, **n** + 8), where **n** is the number of eigenvalues requested. The
    #: default value is None.
    vectors: int | None = None

    #: An Int specifying the maximum number of iterations. The default value is 30.
    maxIterations: int = 30

    #: A SymbolicConstant specifying the size of the Lanczos block steps. The default value is
    #: DEFAULT.
    blockSize: SymbolicConstant = DEFAULT

    #: A SymbolicConstant specifying the maximum number of Lanczos block steps within each
    #: Lanczos run. The default value is DEFAULT.
    maxBlocks: SymbolicConstant = DEFAULT

    #: A SymbolicConstant specifying the method for normalizing eigenvectors. Possible values
    #: are DISPLACEMENT and MASS. The default value is DISPLACEMENT.A value of DISPLACEMENT
    #: indicates normalizing the eigenvectors so that the largest displacement entry in each
    #: vector is unity. A value of MASS indicates normalizing the eigenvectors with respect to
    #: the structure's mass matrix, which results in scaling the eigenvectors so that the
    #: generalized mass for each vector is unity.
    normalization: SymbolicConstant = DISPLACEMENT

    #: None or a Float specifying the frequency at which to evaluate frequency-dependent
    #: properties for viscoelasticity, springs, and dashpots during the eigenvalue extraction.
    #: If the value is None, the analysis product will evaluate the stiffness associated with
    #: frequency-dependent springs and dashpots at zero frequency and will not consider the
    #: stiffness contributions from frequency-domain viscoelasticity in the step. The default
    #: value is None.
    propertyEvaluationFrequency: float | None = None

    #: A Boolean specifying whether to include projection of viscous and structural damping
    #: operators during **AMS** eigenvalue extraction. Valid only when **eigenSolver** = AMS. The
    #: default value is ON.
    projectDamping: Boolean = ON

    #: A SymbolicConstant specifying the type of acoustic-structural coupling in models with
    #: acoustic and structural elements coupled using the TIE option or in models with ASI-type
    #: elements. Possible values are AC_ON, AC_OFF, and AC_PROJECTION. The default value is
    #: AC_ON.
    acousticCoupling: SymbolicConstant = AC_ON

    #: A Float specifying the ratio of the maximum acoustic frequency to the maximum structural
    #: frequency. The default value is 1.0.
    acousticRangeFactor: float = 1

    #: A Boolean specifying whether to add to the damping matrix contributions due to friction
    #: effects. The default value is OFF.
    frictionDamping: Boolean = OFF

    #: A SymbolicConstant specifying the type of matrix storage. Possible values are SYMMETRIC,
    #: UNSYMMETRIC, and SOLVER_DEFAULT. The default value is SOLVER_DEFAULT.
    matrixStorage: SymbolicConstant = SOLVER_DEFAULT

    #: A Boolean specifying whether to activate the SIM-based linear dynamics procedures. The
    #: default value is OFF.
    simLinearDynamics: Boolean = OFF

    #: A Boolean specifying whether to include residual modes from an immediately preceding
    #: Static, Linear Perturbation step. The default value is OFF.
    residualModes: Boolean = OFF

    #: A Float specifying the cutoff frequency for substructure eigenproblems, defined as a
    #: multiplier of the maximum frequency of interest. The default value is 5.0.
    substructureCutoffMultiplier: float = 5

    #: A Float specifying the first cutoff frequency for a reduced eigenproblem, defined as a
    #: multiplier of the maximum frequency of interest. The default value is 1.7.
    firstCutoffMultiplier: float = 1

    #: A Float specifying the second cutoff frequency for a reduced eigenproblem defined as a
    #: multiplier of the maximum frequency of interest. The default value is 1.1.
    secondCutoffMultiplier: float = 1

    #: A String specifying the name of the previous step. The new step appears after this step
    #: in the list of analysis steps.
    previous: str = ""

    #: A String specifying a description of the new step. The default value is an empty string.
    description: str = ""

    #: None or a tuple of Strings specifying the name of a region for which residual modes are
    #: requested. The default value is None.
    residualModeRegion: tuple | None = None

    #: None or a tuple of Ints specifying the degree of freedom for which residual modes are
    #: requested. The default value is None.
    residualModeDof: int | None = None

    #: None or a Region object specifying a region for which eigenvectors should be saved or
    #: the SymbolicConstant None representing the whole model. The default value is None.
    limitSavedEigenvectorRegion: SymbolicConstant

    #: A SymbolicConstant specifying whether the step has an explicit procedure type
    #: (*procedureType* = ANNEAL, DYNAMIC_EXPLICIT, or DYNAMIC_TEMP_DISPLACEMENT).
    explicit: SymbolicConstant

    #: A Boolean specifying whether the step has a perturbation procedure type.
    perturbation: Boolean = OFF

    #: A Boolean specifying whether the step has a mechanical procedure type.
    nonmechanical: Boolean = OFF

    #: A SymbolicConstant specifying the Abaqus procedure. Possible values are:
    #:
    #: - ANNEAL
    #: - BUCKLE
    #: - COMPLEX_FREQUENCY
    #: - COUPLED_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRIC
    #: - DIRECT_CYCLIC
    #: - DYNAMIC_IMPLICIT
    #: - DYNAMIC_EXPLICIT
    #: - DYNAMIC_SUBSPACE
    #: - DYNAMIC_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRICAL_STRUCTURAL
    #: - FREQUENCY
    #: - GEOSTATIC
    #: - HEAT_TRANSFER
    #: - MASS_DIFFUSION
    #: - MODAL_DYNAMICS
    #: - RANDOM_RESPONSE
    #: - RESPONSE_SPECTRUM
    #: - SOILS
    #: - STATIC_GENERAL
    #: - STATIC_LINEAR_PERTURBATION
    #: - STATIC_RIKS
    #: - STEADY_STATE_DIRECT
    #: - STEADY_STATE_MODAL
    #: - STEADY_STATE_SUBSPACE
    #: - VISCO
    procedureType: SymbolicConstant

    #: A Boolean specifying whether the step is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A repository of FieldOutputRequestState objects.
    fieldOutputRequestState: dict[str, FieldOutputRequestState] = {}

    #: A repository of HistoryOutputRequestState objects.
    historyOutputRequestState: dict[str, HistoryOutputRequestState] = {}

    #: A DiagnosticPrint object.
    diagnosticPrint: DiagnosticPrint = DiagnosticPrint()

    #: A Monitor object.
    monitor: Monitor | None = None

    #: A Restart object.
    restart: Restart = Restart()

    #: A repository of AdaptiveMeshConstraintState objects.
    adaptiveMeshConstraintStates: dict[str, AdaptiveMeshConstraintState] = {}

    #: A repository of AdaptiveMeshDomain objects.
    adaptiveMeshDomains: dict[str, AdaptiveMeshDomain] = {}

    #: A Control object.
    control: Control = Control()

    #: A SolverControl object.
    solverControl: SolverControl = SolverControl()

    #: A repository of BoundaryConditionState objects.
    boundaryConditionStates: dict[str, BoundaryConditionState] = {}

    #: A repository of InteractionState objects.
    interactionStates: int | None = None

    #: A repository of LoadState objects.
    loadStates: dict[str, LoadState] = {}

    #: A repository of LoadCase objects.
    loadCases: dict[str, LoadCase] = {}

    #: A repository of PredefinedFieldState objects.
    predefinedFieldStates: dict[str, PredefinedFieldState] = {}

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        previous: str,
        eigensolver: Literal[C.AMS, C.LANCZOS, C.SUBSPACE],
        numEigen: Literal[C.ALL] = ALL,
        description: str = "",
        shift: float = 0,
        minEigen: float | None = None,
        maxEigen: float | None = None,
        vectors: int | None = None,
        maxIterations: int = 30,
        blockSize: Literal[C.DEFAULT] = DEFAULT,
        maxBlocks: Literal[C.DEFAULT] = DEFAULT,
        normalization: Literal[C.MASS, C.DISPLACEMENT] = DISPLACEMENT,
        propertyEvaluationFrequency: float | None = None,
        projectDamping: Boolean = ON,
        acousticCoupling: Literal[C.AC_OFF, C.AC_ON, C.TIE, C.AC_PROJECTION] = AC_ON,
        acousticRangeFactor: float = 1,
        frictionDamping: Boolean = OFF,
        matrixStorage: Literal[C.SYMMETRIC, C.SOLVER_DEFAULT, C.UNSYMMETRIC] = SOLVER_DEFAULT,
        maintainAttributes: Boolean = False,
        simLinearDynamics: Boolean = OFF,
        residualModes: Boolean = OFF,
        substructureCutoffMultiplier: float = 5,
        firstCutoffMultiplier: float = 1,
        secondCutoffMultiplier: float = 1,
        residualModeRegion: str | None = None,
        residualModeDof: str | None = None,
        limitSavedEigenvectorRegion: SymbolicConstant | None = None,
    ):
        """This method creates a FrequencyStep object.

        .. note::
            This function can be accessed by::

                mdb.models[name].FrequencyStep

        Parameters
        ----------
        name
            A String specifying the repository key.
        previous
            A String specifying the name of the previous step. The new step appears after this step
            in the list of analysis steps.
        eigensolver
            A SymbolicConstant specifying the eigensolver. Possible values are LANCZOS, SUBSPACE,
            and AMS.The following optional arguments are ignored unless **eigensolver** = LANCZOS:
            **blockSize**, **maxBlocks**, **normalization**, **propertyEvaluationFrequency**.The following
            optional arguments are ignored unless **eigensolver** = LANCZOS or AMS: **minEigen**,
            **maxEigen**, and **acousticCoupling**.The following optional arguments are ignored unless
            **eigensolver** = AMS: **projectDamping**, **acousticRangeFactor**,
            **substructureCutoffMultiplier**, **firstCutoffMultiplier**, **secondCutoffMultiplier**,
            **residualModeRegion**, **regionalModeDof**, and **limitSavedEigenvectorRegion**.
        numEigen
            The SymbolicConstant ALL or an Int specifying the number of eigenvalues to be calculated
            or ALL. The default value is ALL.
        description
            A String specifying a description of the new step. The default value is an empty string.
        shift
            A Float specifying the shift point in cycles per time. The default value is 0.0.
        minEigen
            None or a Float specifying the minimum frequency of interest in cycles per time. The
            default value is None.
        maxEigen
            None or a Float specifying the maximum frequency of interest in cycles per time. The
            default value is None.
        vectors
            None or an Int specifying the number of vectors used in the iteration. The default is
            the minimum of (2*n*, **n** + 8), where **n** is the number of eigenvalues requested. The
            default value is None.
        maxIterations
            An Int specifying the maximum number of iterations. The default value is 30.
        blockSize
            A SymbolicConstant specifying the size of the Lanczos block steps. The default value is
            DEFAULT.
        maxBlocks
            A SymbolicConstant specifying the maximum number of Lanczos block steps within each
            Lanczos run. The default value is DEFAULT.
        normalization
            A SymbolicConstant specifying the method for normalizing eigenvectors. Possible values
            are DISPLACEMENT and MASS. The default value is DISPLACEMENT.A value of DISPLACEMENT
            indicates normalizing the eigenvectors so that the largest displacement entry in each
            vector is unity. A value of MASS indicates normalizing the eigenvectors with respect to
            the structure's mass matrix, which results in scaling the eigenvectors so that the
            generalized mass for each vector is unity.
        propertyEvaluationFrequency
            None or a Float specifying the frequency at which to evaluate frequency-dependent
            properties for viscoelasticity, springs, and dashpots during the eigenvalue extraction.
            If the value is None, the analysis product will evaluate the stiffness associated with
            frequency-dependent springs and dashpots at zero frequency and will not consider the
            stiffness contributions from frequency-domain viscoelasticity in the step. The default
            value is None.
        projectDamping
            A Boolean specifying whether to include projection of viscous and structural damping
            operators during **AMS** eigenvalue extraction. Valid only when **eigenSolver** = AMS. The
            default value is ON.
        acousticCoupling
            A SymbolicConstant specifying the type of acoustic-structural coupling in models with
            acoustic and structural elements coupled using the TIE option or in models with ASI-type
            elements. Possible values are AC_ON, AC_OFF, and AC_PROJECTION. The default value is
            AC_ON.
        acousticRangeFactor
            A Float specifying the ratio of the maximum acoustic frequency to the maximum structural
            frequency. The default value is 1.0.
        frictionDamping
            A Boolean specifying whether to add to the damping matrix contributions due to friction
            effects. The default value is OFF.
        matrixStorage
            A SymbolicConstant specifying the type of matrix storage. Possible values are SYMMETRIC,
            UNSYMMETRIC, and SOLVER_DEFAULT. The default value is SOLVER_DEFAULT.
        maintainAttributes
            A Boolean specifying whether to retain attributes from an existing step with the same
            name. The default value is False.
        simLinearDynamics
            A Boolean specifying whether to activate the SIM-based linear dynamics procedures. The
            default value is OFF.
        residualModes
            A Boolean specifying whether to include residual modes from an immediately preceding
            Static, Linear Perturbation step. The default value is OFF.
        substructureCutoffMultiplier
            A Float specifying the cutoff frequency for substructure eigenproblems, defined as a
            multiplier of the maximum frequency of interest. The default value is 5.0.
        firstCutoffMultiplier
            A Float specifying the first cutoff frequency for a reduced eigenproblem, defined as a
            multiplier of the maximum frequency of interest. The default value is 1.7.
        secondCutoffMultiplier
            A Float specifying the second cutoff frequency for a reduced eigenproblem defined as a
            multiplier of the maximum frequency of interest. The default value is 1.1.
        residualModeRegion
            None or a sequence of Strings specifying the name of a region for which residual modes
            are requested. The default value is None.
        residualModeDof
            None or a sequence of Ints specifying the degree of freedom for which residual modes are
            requested. The default value is None.
        limitSavedEigenvectorRegion
            None or a Region object specifying a region for which eigenvectors should be saved or
            the SymbolicConstant None representing the whole model. The default value is None.

        Returns
        -------
        FrequencyStep
            A FrequencyStep object.

        Raises
        ------
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        numEigen: Literal[C.ALL] = ALL,
        description: str = "",
        shift: float = 0,
        minEigen: float | None = None,
        maxEigen: float | None = None,
        vectors: int | None = None,
        maxIterations: int = 30,
        blockSize: Literal[C.DEFAULT] = DEFAULT,
        maxBlocks: Literal[C.DEFAULT] = DEFAULT,
        normalization: Literal[C.MASS, C.DISPLACEMENT] = DISPLACEMENT,
        propertyEvaluationFrequency: float | None = None,
        projectDamping: Boolean = ON,
        acousticCoupling: Literal[C.AC_OFF, C.AC_ON, C.TIE, C.AC_PROJECTION] = AC_ON,
        acousticRangeFactor: float = 1,
        frictionDamping: Boolean = OFF,
        matrixStorage: Literal[C.SYMMETRIC, C.SOLVER_DEFAULT, C.UNSYMMETRIC] = SOLVER_DEFAULT,
        simLinearDynamics: Boolean = OFF,
        residualModes: Boolean = OFF,
        substructureCutoffMultiplier: float = 5,
        firstCutoffMultiplier: float = 1,
        secondCutoffMultiplier: float = 1,
        residualModeRegion: str | None = None,
        residualModeDof: str | None = None,
        limitSavedEigenvectorRegion: SymbolicConstant | None = None,
    ):
        """This method modifies the FrequencyStep object.

        Parameters
        ----------
        numEigen
            The SymbolicConstant ALL or an Int specifying the number of eigenvalues to be calculated
            or ALL. The default value is ALL.
        description
            A String specifying a description of the new step. The default value is an empty string.
        shift
            A Float specifying the shift point in cycles per time. The default value is 0.0.
        minEigen
            None or a Float specifying the minimum frequency of interest in cycles per time. The
            default value is None.
        maxEigen
            None or a Float specifying the maximum frequency of interest in cycles per time. The
            default value is None.
        vectors
            None or an Int specifying the number of vectors used in the iteration. The default is
            the minimum of (2*n*, **n** + 8), where **n** is the number of eigenvalues requested. The
            default value is None.
        maxIterations
            An Int specifying the maximum number of iterations. The default value is 30.
        blockSize
            A SymbolicConstant specifying the size of the Lanczos block steps. The default value is
            DEFAULT.
        maxBlocks
            A SymbolicConstant specifying the maximum number of Lanczos block steps within each
            Lanczos run. The default value is DEFAULT.
        normalization
            A SymbolicConstant specifying the method for normalizing eigenvectors. Possible values
            are DISPLACEMENT and MASS. The default value is DISPLACEMENT.A value of DISPLACEMENT
            indicates normalizing the eigenvectors so that the largest displacement entry in each
            vector is unity. A value of MASS indicates normalizing the eigenvectors with respect to
            the structure's mass matrix, which results in scaling the eigenvectors so that the
            generalized mass for each vector is unity.
        propertyEvaluationFrequency
            None or a Float specifying the frequency at which to evaluate frequency-dependent
            properties for viscoelasticity, springs, and dashpots during the eigenvalue extraction.
            If the value is None, the analysis product will evaluate the stiffness associated with
            frequency-dependent springs and dashpots at zero frequency and will not consider the
            stiffness contributions from frequency-domain viscoelasticity in the step. The default
            value is None.
        projectDamping
            A Boolean specifying whether to include projection of viscous and structural damping
            operators during **AMS** eigenvalue extraction. Valid only when **eigenSolver** = AMS. The
            default value is ON.
        acousticCoupling
            A SymbolicConstant specifying the type of acoustic-structural coupling in models with
            acoustic and structural elements coupled using the TIE option or in models with ASI-type
            elements. Possible values are AC_ON, AC_OFF, and AC_PROJECTION. The default value is
            AC_ON.
        acousticRangeFactor
            A Float specifying the ratio of the maximum acoustic frequency to the maximum structural
            frequency. The default value is 1.0.
        frictionDamping
            A Boolean specifying whether to add to the damping matrix contributions due to friction
            effects. The default value is OFF.
        matrixStorage
            A SymbolicConstant specifying the type of matrix storage. Possible values are SYMMETRIC,
            UNSYMMETRIC, and SOLVER_DEFAULT. The default value is SOLVER_DEFAULT.
        simLinearDynamics
            A Boolean specifying whether to activate the SIM-based linear dynamics procedures. The
            default value is OFF.
        residualModes
            A Boolean specifying whether to include residual modes from an immediately preceding
            Static, Linear Perturbation step. The default value is OFF.
        substructureCutoffMultiplier
            A Float specifying the cutoff frequency for substructure eigenproblems, defined as a
            multiplier of the maximum frequency of interest. The default value is 5.0.
        firstCutoffMultiplier
            A Float specifying the first cutoff frequency for a reduced eigenproblem, defined as a
            multiplier of the maximum frequency of interest. The default value is 1.7.
        secondCutoffMultiplier
            A Float specifying the second cutoff frequency for a reduced eigenproblem defined as a
            multiplier of the maximum frequency of interest. The default value is 1.1.
        residualModeRegion
            None or a sequence of Strings specifying the name of a region for which residual modes
            are requested. The default value is None.
        residualModeDof
            None or a sequence of Ints specifying the degree of freedom for which residual modes are
            requested. The default value is None.
        limitSavedEigenvectorRegion
            None or a Region object specifying a region for which eigenvectors should be saved or
            the SymbolicConstant None representing the whole model. The default value is None.

        Raises
        ------
        RangeError
        """
        ...
